@@ -24,6 +24,7 @@
 
 ;;; Code:
 
+(require 'riece-version)
 (require 'riece-globals)
 
 ;; User modifiable variables.
@@ -109,7 +110,8 @@ way is to put Riece variables on .emacs or file loaded from there."
 			  riece-url
 			  riece-button
 			  riece-menu
-			  riece-icon)
+			  riece-icon
+			  riece-ignore)
   "Add-ons insinuated into Riece."
   :type '(repeat symbol)
   :group 'riece-options)
@@ -192,6 +194,11 @@ way is to put Riece variables on .emacs or file loaded from there."
 			 (list (string :tag "Channel") (string :tag "Key"))))
   :group 'riece-channel)
 
+(defcustom riece-startup-server-list nil
+  "A list of servers to connect automatically at startup."
+  :type '(repeat (string :tag "Server"))
+  :group 'riece-server)
+
 (defcustom riece-retry-with-new-nickname nil
   "When nickname has already been in use, grow-tail automatically."
   :type 'boolean
@@ -203,6 +210,11 @@ way is to put Riece variables on .emacs or file loaded from there."
 		(const nil))
   :group 'riece-server)
 
+(defcustom riece-default-open-connection-function #'open-network-stream
+  "Default function used for connecting to an IRC server."
+  :type 'function
+  :group 'riece-server)
+
 (defcustom riece-channel-buffer-mode t
   "When non-nil, Riece will display a channel buffer."
   :type 'boolean
@@ -234,14 +246,14 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type 'boolean
   :group 'riece-options)
 
-(defcustom riece-quit-message nil
+(defcustom riece-quit-message (riece-extended-version)
   "Default quit message."
-  :type '(radio (string :tag "Quit message"))
+  :type '(string :tag "Quit message")
   :group 'riece-options)
 
-(defcustom riece-away-message nil
+(defcustom riece-away-message "Gone"
   "Default away message."
-  :type '(radio (string :tag "Away message"))
+  :type '(string :tag "Away message")
   :group 'riece-options)
 
 (defcustom riece-gather-channel-modes nil
