@@ -29,6 +29,7 @@
 (require 'riece-server)
 (require 'riece-compat)
 (require 'riece-commands)
+(require 'riece-addon)
 
 (autoload 'derived-mode-class "derived")
 
@@ -53,7 +54,7 @@
 (put 'riece-channel-mode 'derived-mode-parent 'riece-dialogue-mode)
 
 (defvar riece-buffer-alist
-  '((riece-command-buffer "*Commands*" riece-command-mode)
+  '((riece-command-buffer "*Command*" riece-command-mode)
     (riece-dialogue-buffer "*Dialogue*" riece-dialogue-mode)
     (riece-others-buffer "*Others*" riece-others-mode)
     (riece-channel-list-buffer "*Channels*" riece-channel-list-mode)
@@ -62,6 +63,9 @@
 (defvar riece-shrink-buffer-idle-timer nil
   "Timer object to periodically shrink channel buffers.")
 
+(defvar riece-addons-insinuated nil
+  "Non nil if add-ons are already insinuated.")
+
 (defvar riece-select-keys
   `("1" riece-command-switch-to-channel-by-number-1
     "2" riece-command-switch-to-channel-by-number-2
@@ -151,6 +155,7 @@ If optional argument SAFE is nil, overwrite previous definitions."
     "/" riece-command-raw
     ">" end-of-buffer
     "<" beginning-of-buffer
+    "^" riece-command-list-addons
     "\C-ta" riece-command-toggle-away
     "c" riece-command-select-command-buffer
     "f" riece-command-finger
@@ -180,7 +185,8 @@ If optional argument SAFE is nil, overwrite previous definitions."
 
   (riece-define-keys riece-command-mode-map
     "\r" riece-command-enter-message
-    [(control return)] riece-command-enter-message-as-notice)
+    [(control return)] riece-command-enter-message-as-notice
+    [tab] riece-command-complete-user)
 
   (riece-define-keys (riece-command-map riece-command-prefix
 					riece-command-mode-map)
@@ -261,10 +267,16 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
   (interactive "P")
   (riece-read-variables-files (if noninteractive
 				  (car command-line-args-left)))
-  (riece-insinuate-addons riece-addons)
   (run-hooks 'riece-after-load-startup-hook)
   (if (riece-server-opened)
       (riece-command-configure-windows)
+    (unless riece-addons-insinuated
+      (setq riece-addons (riece-resolve-addons riece-addons))
+      (let ((pointer riece-addons))
+	(while pointer
+	  (riece-insinuate-addon (car pointer) riece-debug)
+	  (setq pointer (cdr pointer))))
+      (setq riece-addons-insinuated t))
     (if (or confirm (null riece-server))
 	(setq riece-server (completing-read "Server: " riece-server-alist)))
     (if (stringp riece-server)
@@ -291,6 +303,11 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
       (while server-list
 	(riece-command-open-server (car server-list))
 	(setq server-list (cdr server-list))))
+    (let ((pointer riece-addons))
+      (while pointer
+	(unless (get (car pointer) 'riece-addon-default-disabled)
+	  (riece-enable-addon (car pointer) riece-debug))
+	(setq pointer (cdr pointer))))
     (run-hooks 'riece-startup-hook)
     (message "%s" (substitute-command-keys
 		   "Type \\[describe-mode] for help"))))
@@ -323,6 +340,7 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
   (setq riece-server nil
 	riece-current-channels nil
 	riece-current-channel nil
+	riece-channel-buffer nil
 	riece-channel-buffer-alist nil
 	riece-user-indicator nil
 	riece-long-channel-indicator "None"
@@ -345,13 +363,12 @@ For a list of the generic commands type \\[riece-command-generic] ? RET.
   (setq riece-away-indicator "-"
 	riece-operator-indicator "-"
 	major-mode 'riece-command-mode
-	mode-name "Commands"
+	mode-name "Command"
 	mode-line-buffer-identification
 	(riece-mode-line-buffer-identification
 	 '("Riece: "
 	   riece-away-indicator
 	   riece-operator-indicator
-	   riece-freeze-indicator
 	   " "
 	   riece-user-indicator
 	   " "
@@ -376,6 +393,7 @@ Instead, these commands are available:
 \\{riece-dialogue-mode-map}"
   (kill-all-local-variables)
   (make-local-variable 'riece-freeze)
+  (make-local-variable 'riece-freeze-indicator)
   (setq riece-freeze riece-default-freeze
 	riece-away-indicator "-"
 	riece-operator-indicator "-"
@@ -467,75 +485,7 @@ Instead, these commands are available:
 		    (eq major-mode (nth 2 (car alist))))
 	  (funcall (nth 2 (car alist))))
 	(setq alist (cdr alist))))))
-
-(defun riece-load-and-build-addon-dependencies (addons)
-  (let ((load-path (cons riece-addon-directory load-path))
-	dependencies)
-    (while addons
-      (require (car addons))		;error will be reported here
-      (let* ((requires
-	      (funcall (or (intern-soft
-			    (concat (symbol-name (car addons)) "-requires"))
-			   #'ignore)))
-	     (pointer requires)
-	     entry)
-	;; Increment succs' pred count.
-	(if (setq entry (assq (car addons) dependencies))
-	    (setcar (cdr entry) (+ (length requires) (nth 1 entry)))
-	  (setq dependencies (cons (list (car addons) (length requires))
-				   dependencies)))
-	;; Merge pred's succs.
-	(while pointer
-	  (if (setq entry (assq (car pointer) dependencies))
-	      (setcdr (cdr entry)
-		      (cons (car addons) (nthcdr 2 entry)))
-	    (setq dependencies (cons (list (car pointer) 0 (car addons))
-				     dependencies)))
-	  (setq pointer (cdr pointer))))
-      (setq addons (cdr addons)))
-    dependencies))
-
-(defun riece-insinuate-addons (addons)
-  (let ((pointer addons)
-	dependencies queue)
-    ;; Uniquify, first.
-    (while pointer
-      (if (memq (car pointer) (cdr pointer))
-	  (setcar pointer nil))
-      (setq pointer (cdr pointer)))
-    (setq dependencies (riece-load-and-build-addon-dependencies
-			(delq nil addons))
-	  pointer dependencies)
-    ;; Sort them.
-    (while pointer
-      (if (zerop (nth 1 (car pointer)))
-	  (setq dependencies (delq (car pointer) dependencies)
-		queue (cons (car pointer) queue)))
-      (setq pointer (cdr pointer)))
-    (setq addons nil)
-    (while queue
-      (setq addons (cons (car (car queue)) addons)
-	    pointer (nthcdr 2 (car queue)))
-      (while pointer
-	(let* ((entry (assq (car pointer) dependencies))
-	       (count (1- (nth 1 entry))))
-	  (if (zerop count)
-	      (progn
-		(setq dependencies (delq entry dependencies)
-		      queue (nconc queue (list entry))))
-	    (setcar (cdr entry) count)))
-	(setq pointer (cdr pointer)))
-      (setq queue (cdr queue)))
-    (if dependencies
-	(error "Circular add-on dependency found"))
-    (setq addons (nreverse addons))
-    (while addons
-      (require (car addons))		;implicit dependency
-      (funcall (intern (concat (symbol-name (car addons)) "-insinuate")))
-      (if riece-debug
-	  (message "Add-on %S is loaded" (car addons)))
-      (setq addons (cdr addons)))))
-
+      
 (provide 'riece)
 
 ;;; riece.el ends here
